from requests import Session
from requests import HTTPError
from json.decoder import JSONDecodeError
from . import API_KEY

from .exceptions import APIError
from .urls import Urls

DATA = "data"


class Pool(object):
    __session = None
    __api_key = None

    def __init__(self, coin_name, api_key=API_KEY):
        self.__api_key = api_key
        self.coin_name = coin_name
        self.urls = Urls()

    @property
    def session(self):
        if self.__session is None:
            self.__session = Session()
            self.__session.params = {"api_key": self.__api_key}

        return self.__session

    @session.setter
    def session(self, value):
        raise AttributeError("Setting 'session' attribute is prohibited.")

    @staticmethod
    def __to_json(response):
        """Private method to call json method on response object

        Parameters
        ----------
        response : Response
            The response object

        Returns
        -------
        dict
            JSON response represented as a Python dictionary
        """
        return response.json()

    def __get_data(self, url):
        """Private method to make a GET request to the URL

        Parameters
        ----------
        url : str
            The URL to query

        Returns
        -------
        dict
            JSON response represented as a Python dictionary

        Raises
        ------
        HTTPError
            Raises on HTTP Error
        JSONDecodeError
            Raises when there is an issue parsing the JSON response
        """
        try:
            response = self.session.get(url)

            # raises if the status code is an error - 4xx, 5xx
            response.raise_for_status()

            return self.__to_json(response)
        except HTTPError as e:
            pass
        except JSONDecodeError as e:
            pass

    def get_block_count(self):
        """ "Get current block height in blockchain

        Returns
        -------
        int
            block count
        """
        return int(
            self.__get_data(self.urls.get_block_count_url(pool=self.coin_name))[
                self.urls.action_get_block_count
            ][DATA]
        )

    def get_block_stats(self):
        """ "Get pool block stats

        Returns
        -------
        dict
            block stats
        """
        return self.__get_data(self.urls.get_block_stats_url(pool=self.coin_name))[
            self.urls.action_get_block_stats
        ][DATA]

    def get_blocks_found(self):
        """Get last N blocks found as configured in admin panel

        Returns
        -------
        list of dict:
            data for the last N blocks found
        """
        return self.__get_data(self.urls.get_blocks_found_url(pool=self.coin_name))[
            self.urls.action_get_blocks_found
        ][DATA]

    def get_current_workers(self):
        """Get the total hash rate of current workers for a coin pool

        Returns
        -------
        int
            the hash rate in kH/s
        """
        return int(
            self.__get_data(self.urls.get_current_workers_url(pool=self.coin_name))[
                self.urls.action_get_current_workers
            ][DATA]
        )

    def get_dashboard(self):
        """Load a user's dashboard data for a pool: hash rate, share rate, balance, recent credits

        Returns
        -------
        dict
            dashboard data
        """
        return self.__get_data(self.urls.get_dashboard_data_url(pool=self.coin_name))[
            self.urls.action_get_dashboard_data
        ][DATA]

    def get_difficulty(self):
        """Get current difficulty in blockchain

        Returns
        -------
        int
            network difficulty
        """
        return int(
            self.__get_data(self.urls.get_difficulty_url(pool=self.coin_name))[
                self.urls.action_get_difficulty
            ][DATA]
        )

    def get_estimated_time(self):
        """Get estimated time to next block based on pool hashrate (seconds)

        Returns
        -------
        int
            estimated time until next block in seconds
        """
        return int(
            self.__get_data(self.urls.get_estimated_time_url(pool=self.coin_name))[
                self.urls.action_get_estimated_time
            ][DATA]
        )

    def get_hourly_hash_rate(self):
        """Get the average hash rate each hour for the last 24 hours, total and by worker, currently broken
        according to API docs

        Returns
        -------
        list of dict
            the first entry in the list is total hashrate, all following entries are for each worker
        """
        return self.__get_data(
            self.urls.get_hourly_hash_rates_url(pool=self.coin_name)
        )[self.urls.action_get_hourly_hash_rates][DATA]["mine"]

    def get_nav_bar_data(self):
        """Get the data displayed on the navbar. Always returns { "error": "disabled" }

        Returns
        -------
        dict of str
            error message
        """
        return self.__get_data(self.urls.get_nav_bar_data_url(pool=self.coin_name))[
            self.urls.action_get_nav_bar_data
        ][DATA]

    def get_pool_hash_rate(self):
        """Get current pool hash rate

        Returns
        -------
        float
            current pool hash rate in kH/s
        """
        return self.__get_data(self.urls.get_pool_hash_rate_url(pool=self.coin_name))[
            self.urls.action_get_pool_hash_rate
        ][DATA]

    def get_pool_info(self):
        """Get the information on pool settings

        Returns
        -------
        dict
            pool settings
        """
        return self.__get_data(self.urls.get_pool_info_url(pool=self.coin_name))[
            self.urls.action_get_pool_info
        ][DATA]

    def get_pool_share_rate(self):
        """Get current pool share rate (shares/s)

        Returns
        -------
        int
            seems to always be 0
        """
        return self.__get_data(self.urls.get_pool_share_rate_url(pool=self.coin_name))[
            self.urls.action_get_pool_share_rate
        ]

    def get_pool_status(self):
        """Fetch overall pool status

        Returns
        -------
        dict
            pool status as a dict
        """
        return self.__get_data(self.urls.get_pool_status_url(pool=self.coin_name))[
            self.urls.action_get_pool_status
        ][DATA]

    def get_time_since_last_block(self):
        """Get time since last block found (seconds)

        Returns
        -------
        int
            time since last block found in seconds
        """
        return self.__get_data(
            self.urls.get_time_since_last_block_url(pool=self.coin_name)
        )[self.urls.action_get_time_since_last_block][DATA]

    def get_top_contributors(self):
        """Fetch top contributors data

        Returns
        -------
        dict
            returns account and hash rate as a dict
        """
        return self.__get_data(self.urls.get_top_contributors_url(pool=self.coin_name))[
            self.urls.action_get_top_contributors
        ][DATA]["hashes"]

    def get_user_balance(self):
        """Fetch a user's balance

        Returns
        -------
        dict of float
            returns confirmed and unconfirmed balances as a dict
        """
        return self.__get_data(self.urls.get_user_balance_url(pool=self.coin_name))[
            self.urls.action_get_user_balance
        ][DATA]

    def get_user_hash_rate(self):
        """Fetch a user's total hash rate

        Returns
        -------
        float
            total hash rate in kH/s
        """
        return self.__get_data(self.urls.get_user_hash_rate_url(pool=self.coin_name))[
            self.urls.action_get_user_hash_rate
        ][DATA]

    def get_user_share_rate(self):
        """Fetch a user's share rate

        Returns
        -------
        int
            seems to always be 0
        """
        return self.__get_data(self.urls.get_user_share_rate_url(pool=self.coin_name))[
            self.urls.action_get_user_share_rate
        ][DATA]

    def get_user_status(self):
        """Fetch a user's overall status

        Returns
        -------
        dict
            user status info: username, shares[valid|invalid|id|donate_percent|is_anonymous|username],
            hash rate, and share rate
        """
        return self.__get_data(self.urls.get_user_status_url(pool=self.coin_name))[
            self.urls.action_get_user_status
        ][DATA]

    def get_user_transactions(self):
        """Get a user's transactions

        Returns
        -------
        list of dict
            data on up to the last 30 transactions for a user on a pool
        """
        return self.__get_data(
            self.urls.get_user_transactions_url(pool=self.coin_name)
        )[self.urls.action_get_user_transactions][DATA]["transactions"]

    def get_user_workers(self):
        """Fetch a user's worker status

        Returns
        -------
        list of dict
            data on each worker represented as a dict: id, username, password, monitor, hash rate, difficulty
        """
        return self.__get_data(self.urls.get_user_workers_url(pool=self.coin_name))[
            self.urls.action_get_user_workers
        ][DATA]

    def public(self):
        """Fetch public pool statistics, no authentication required

        Returns
        -------
        dict
            pool_name, hashrate, workers, shares_this_round, last_block, network_hashrate
        """
        return self.__get_data(self.urls.public_url(self.coin_name))

    def get_auto_switching_and_profits_statistics(self):
        """Get auto switching information for all algorithms

        Returns
        -------
        list of dict
            get list of auto switching statistics for each algorithm as a dict
        """
        path = self.urls.get_auto_switching_and_profits_statistics_url()
        response = self.__get_data(path)
        if response["success"] is not True:
            raise APIError("Call failed")

        return response["return"]

    def get_mining_profit_and_statistics(self):
        """Get mining profits statistics

        Returns
        -------
        list of dict
            mining statistics for each coin
        """
        path = self.urls.get_mining_profit_and_statistics_url()
        response = self.__get_data(path)
        if response["success"] is not True:
            raise APIError("Call failed")

        return response["return"]

    def get_user_all_balances(self):
        """Get all currency balances for a user

        Returns
        -------
        list of dict
            balances for each coin
        """
        return self.__get_data(self.urls.get_user_all_balances_url())[
            self.urls.action_get_user_all_balances
        ][DATA]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['miningpoolhub_py']

package_data = \
{'': ['*']}

install_requires = \
['python-dotenv>=0.19.1,<0.20.0', 'requests>=2.26.0,<3.0.0']

extras_require = \
{'docs': ['Sphinx>=4.2.0,<5.0.0',
          'sphinx-rtd-theme>=1.0.0,<2.0.0',
          'sphinxcontrib-napoleon>=0.7,<0.8']}

setup_kwargs = {
    'name': 'miningpoolhub-py',
    'version': '0.1.12',
    'description': 'A Python wrapper for the Mining Pool Hub REST API',
    'long_description': "\nminingpoolhub_py\n============================================\nA Python wrapper for the Mining Pool Hub REST API\n\nInstallation\n------------\nInstall with pip:\n\n.. code-block:: bash\n\n   pip install miningpoolhub_py\n\nUsage\n------------\n\nUniversal Endpoints\n-------------------\nMining Pool Hub supports auto switching between coins. Obtain statistics for all coins with the following methods\n\n.. code-block:: python\n\n   from miningpoolhub_py import Pool\n\n   pool_instance = Pool('ethereum')\n   pool_instance.get_all_user_balances()\n   pool_instance.get_auto_switching_and_profits_statistics()\n   pool_instance.get_mining_profit_and_statistics()\n\n\nPool Selection\n-------------------\nMining Pool Hub has different base urls for each coin they offer. Create a new pool object for every coin you are\ninterested in mining statistics for\n\n.. code-block:: python\n\n   from miningpoolhub_py import Pool\n   pool_instance = Pool('ethereum')\n   pool_instance.get_dashboard()\n\nAuthentication\n-------------------\n\nEnvironment File\n--------------------------------\n.. code-block::\n\n   MPH_API_KEY=<api_key>\n\nPass API Key to Pool Constructor\n--------------------------------\n.. code-block:: python\n\n   from miningpoolhub_py import Pool\n   pool_instance = Pool('ethereum', '<api_key>')\n\nReferences\n------------\n\nMining Pool Hub\n---------------------------------------------\n- `Mining Pool Hub <https://miningpoolhub.com/>`_\n- `API Reference <https://github.com/miningpoolhub/php-mpos/wiki/API-Reference>`_\n- `Mining Pool Hub API Key <https://miningpoolhub.com/?page=account&action=edit>`_\n\nPython API Wrapper & and CI/CD Pipeline\n---------------------------------------------\n- `Building and Testing an API Wrapper in Python <https://semaphoreci.com/community/tutorials/building-and-testing-an-api-wrapper-in-python>`_\n- `Creating a Python API Wrapper \\(Ally Invest API\\) <https://medium.com/analytics-vidhya/creating-a-python-api-wrapper-ally-invest-api-568934a1411c>`_\n- `Publishing a Package to PyPI with Poetry <https://www.ianwootten.co.uk/2020/10/20/publishing-a-package-to-pypi-with-poetry/>`_\n- `Publishing to PyPI Using GitHub Actions <https://www.ianwootten.co.uk/2020/10/23/publishing-to-pypi-using-github-actions/>`_\n   - `Code Repo <https://github.com/niftydigits/ftrack-s3-accessor/tree/master/.github/workflows>`_\n\nContribute\n----------\n\n- `Issue Tracker <https://github.com/CoryKrol/miningpoolhub_py/issues>`_\n- `Source Code <https://github.com/CoryKrol/miningpoolhub_py>`_\n\nSupport\n-------\n\n`Open an issue <https://github.com/CoryKrol/miningpoolhub_py/issues/new>`_\n\nLicense\n-------\n\nThe project is licensed under the Apache 2 license",
    'author': 'CoryKrol',
    'author_email': '16892390+CoryKrol@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/CoryKrol/miningpoolhub_py',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



class Urls:
    def __init__(self):
        self.protocol = "https://"
        self.base_url = "miningpoolhub.com/index.php?page=api&action={action}"
        self.coin_pool = "{coin_pool}."
        self.no_pool_base_url = self.protocol + self.base_url

        # API Actions
        self.action_get_block_count = "getblockcount"
        self.action_get_block_stats = "getblockstats"
        self.action_get_blocks_found = "getblocksfound"
        self.action_get_current_workers = "getcurrentworkers"
        self.action_get_dashboard_data = "getdashboarddata"
        self.action_get_difficulty = "getdifficulty"
        self.action_get_estimated_time = "getestimatedtime"
        self.action_get_hourly_hash_rates = "gethourlyhashrates"
        self.action_get_nav_bar_data = "getnavbardata"
        self.action_get_pool_hash_rate = "getpoolhashrate"
        self.action_get_pool_info = "getpoolinfo"
        self.action_get_pool_share_rate = "getpoolsharerate"
        self.action_get_pool_status = "getpoolstatus"
        self.action_get_time_since_last_block = "gettimesincelastblock"
        self.action_get_top_contributors = "gettopcontributors"
        self.action_get_user_balance = "getuserbalance"
        self.action_get_user_hash_rate = "getuserhashrate"
        self.action_get_user_share_rate = "getusersharerate"
        self.action_get_user_status = "getuserstatus"
        self.action_get_user_transactions = "getusertransactions"
        self.action_get_user_workers = "getuserworkers"
        self.action_public = "public"
        self.action_get_auto_switching_and_profits_statistics = (
            "getautoswitchingandprofitsstatistics"
        )
        self.action_get_mining_and_profits_statistics = "getminingandprofitsstatistics"
        self.action_get_user_all_balances = "getuserallbalances"

        # Urls specific to each coins' mining pool
        self.get_block_count = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_block_count)
        )
        self.get_block_stats = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_block_stats)
        )
        self.get_blocks_found = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_blocks_found)
        )
        self.get_current_workers = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_current_workers)
        )
        self.get_dashboard_data = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_dashboard_data)
        )
        self.get_difficulty = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_difficulty)
        )
        self.get_estimated_time = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_estimated_time)
        )
        self.get_hourly_hash_rates = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_hourly_hash_rates)
        )
        self.get_nav_bar_data = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_nav_bar_data)
        )
        self.get_pool_hash_rate = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_pool_hash_rate)
        )
        self.get_pool_info = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_pool_info)
        )
        self.get_pool_share_rate = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_pool_share_rate)
        )
        self.get_pool_status = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_pool_status)
        )
        self.get_time_since_last_block = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_time_since_last_block)
        )
        self.get_top_contributors = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_top_contributors)
        )
        self.get_user_balance = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_user_balance)
        )
        self.get_user_hash_rate = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_user_hash_rate)
        )
        self.get_user_share_rate = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_user_share_rate)
        )
        self.get_user_status = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_user_status)
        )
        self.get_user_transactions = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_user_transactions)
        )
        self.get_user_workers = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_get_user_workers)
        )
        self.public = (
            self.protocol
            + self.coin_pool
            + self.base_url.format(action=self.action_public)
        )

        # Info for all mining pools
        self.get_auto_switching_and_profits_statistics = self.no_pool_base_url.format(
            action=self.action_get_auto_switching_and_profits_statistics
        )
        self.get_mining_profit_and_statistics = self.no_pool_base_url.format(
            action=self.action_get_mining_and_profits_statistics
        )
        self.get_user_all_balances = self.no_pool_base_url.format(
            action=self.action_get_user_all_balances
        )

    def base_url(self):
        return self.base_url

    def get_block_count_url(self, pool):
        return self.get_block_count.format(coin_pool=pool)

    def get_block_stats_url(self, pool):
        return self.get_block_stats.format(coin_pool=pool)

    def get_blocks_found_url(self, pool):
        return self.get_blocks_found.format(coin_pool=pool)

    def get_current_workers_url(self, pool):
        return self.get_current_workers.format(coin_pool=pool)

    def get_dashboard_data_url(self, pool):
        return self.get_dashboard_data.format(coin_pool=pool)

    def get_difficulty_url(self, pool):
        return self.get_difficulty.format(coin_pool=pool)

    def get_estimated_time_url(self, pool):
        return self.get_estimated_time.format(coin_pool=pool)

    def get_hourly_hash_rates_url(self, pool):
        return self.get_hourly_hash_rates.format(coin_pool=pool)

    def get_nav_bar_data_url(self, pool):
        return self.get_nav_bar_data.format(coin_pool=pool)

    def get_pool_hash_rate_url(self, pool):
        return self.get_pool_hash_rate.format(coin_pool=pool)

    def get_pool_info_url(self, pool):
        return self.get_pool_info.format(coin_pool=pool)

    def get_pool_share_rate_url(self, pool):
        return self.get_pool_share_rate.format(coin_pool=pool)

    def get_pool_status_url(self, pool):
        return self.get_pool_status.format(coin_pool=pool)

    def get_time_since_last_block_url(self, pool):
        return self.get_time_since_last_block.format(coin_pool=pool)

    def get_top_contributors_url(self, pool):
        return self.get_top_contributors.format(coin_pool=pool)

    def get_user_balance_url(self, pool):
        return self.get_user_balance.format(coin_pool=pool)

    def get_user_hash_rate_url(self, pool):
        return self.get_user_hash_rate.format(coin_pool=pool)

    def get_user_share_rate_url(self, pool):
        return self.get_user_share_rate.format(coin_pool=pool)

    def get_user_status_url(self, pool):
        return self.get_user_status.format(coin_pool=pool)

    def get_user_transactions_url(self, pool):
        return self.get_user_transactions.format(coin_pool=pool)

    def get_user_workers_url(self, pool):
        return self.get_user_workers.format(coin_pool=pool)

    def public_url(self, pool):
        return self.public.format(coin_pool=pool)

    def get_auto_switching_and_profits_statistics_url(self):
        return self.get_auto_switching_and_profits_statistics

    def get_mining_profit_and_statistics_url(self):
        return self.get_mining_profit_and_statistics

    def get_user_all_balances_url(self):
        return self.get_user_all_balances
